#include "Menu.h"

#include <string>

void Menu::InitMenu(Sandbox& sandbox)
{
	CreateBorder(sandbox);

	const int x = SCRWIDTH - MENUSIZE;

	int i = 0;
	for (auto type : types)
	{
		CreateBox(sandbox, type, glm::ivec2(x + 25, 25 + (55 * i)));
		CreateText(sandbox, type, glm::ivec2(x + 35, 35 + (55 * i)), EnumString[i]);
		i++;
	}

	CreateBox (sandbox, Types::Stone, glm::ivec2(x + 25, 25 + (55 * 15)));
	CreateText(sandbox, Types::Stone, glm::ivec2(x + 35, 35 + (55 * 15)), "block");

	CreateBox (sandbox, Types::Stone, glm::ivec2(x + 25, 25 + (55 * 16)));
	CreateText(sandbox, Types::Stone, glm::ivec2(x + 35, 35 + (55 * 16)), "delete");

	CreateBox (sandbox, Types::Stone, glm::ivec2(x + 25, 25 + (55 * 17)));
	CreateText(sandbox, Types::Stone, glm::ivec2(x + 35, 35 + (55 * 17)), "reset");

	CreateBox(sandbox, Types::Stone, glm::ivec2(x + 25, 25 + (55 * 18)));
	CreateText(sandbox, Types::Stone, glm::ivec2(x + 35, 35 + (55 * 18)), "exit");
}

bool Menu::CheckMouse(Sandbox& sandbox, glm::ivec2 Mouse)
{
	const int xMin = SCRWIDTH - MENUSIZE + 25;
	const int xMax = SCRWIDTH - 25;
	const int yMin = 25;

	int i = 0;
	for (auto type : types)
	{
		if (Mouse.x > xMin && Mouse.x < xMax && Mouse.y > yMin + 55 * i && Mouse.y < yMin + 55 * i + 50)
			sandbox.draw = type;
		i++;
	}

	if (Mouse.x > xMin && Mouse.x < xMax && Mouse.y > yMin + 55 * 15 && Mouse.y < yMin + 55 * 15 + 50)
		sandbox.draw = Types::Block;

	if (Mouse.x > xMin && Mouse.x < xMax && Mouse.y > yMin + 55 * 16 && Mouse.y < yMin + 55 * 16 + 50)
		sandbox.draw = Types::Air;

	if (Mouse.x > xMin && Mouse.x < xMax && Mouse.y > yMin + 55 * 17 && Mouse.y < yMin + 55 * 17 + 50)
	{
		// reset
		sandbox.cells.clear();
		for (int i = 0; i < SCRHEIGHT * SCRWIDTH; i++)
		{
			sandbox.grid[i] = nullptr;
		}

		// init border
		for (int y = 0; y < SCRHEIGHT; y++)
		{
			for (int x = 0; x < SCRWIDTH; x++)
			{
				if (x < BORDERSIZE || x > SCRWIDTH - BORDERSIZE || y < BORDERSIZE || y > SCRHEIGHT - BORDERSIZE)
					sandbox.cells.push_back(new Update(sandbox, Types::Block, glm::ivec2(x, y), true));
			}
		}
	}

	if (Mouse.x > xMin && Mouse.x < xMax && Mouse.y > yMin + 55 * 18 && Mouse.y < yMin + 55 * 18 + 50)
		return true;

	return false;
}

void Menu::CreateBorder(Sandbox& sandbox)
{
	int xMin = SCRWIDTH - MENUSIZE;
	int size = 5;

	for (int y = 0; y < SCRHEIGHT; y++)
	{
		for (int x = xMin; x < SCRWIDTH; x++)
		{
			if (x < xMin + size || x > SCRWIDTH - size || y < size || y > SCRHEIGHT - size)
				sandbox.Menu.push_back(new Update(sandbox, Types::Block, glm::ivec2(x, y), false));
		}
	}
}

void Menu::CreateBox(Sandbox& sandbox, Types type, glm::ivec2 pos)
{
	const int xMax = SCRWIDTH - 25;
	const int yMax = pos.y + 50;
	const int size = 5;

	for (int y = pos.y; y <= yMax; y++)
	{
		for (int x = pos.x; x <= xMax; x++)
		{
			if (x < pos.x + size || x > xMax - size || y < pos.y + size || y > yMax - size)
				sandbox.Menu.push_back(new Update(sandbox, type, glm::ivec2(x, y), false));
		}
	}
}

void Menu::CreateText(Sandbox& sandbox, Types type, glm::ivec2 pos, const char* text)
{
	int counter = 0;

	while (text[counter] != 0)
	{
		CreateLetter(sandbox, type, glm::ivec2(pos.x + (counter * TEXTSIZE * 6), pos.y), text[counter]);
		counter++;
	}
}

void Menu::CreateLetter(Sandbox& sandbox, Types type, glm::ivec2 pos, const char text)
{
	std::string pixel;
	const int width = 5 + 2;

	switch (text)
	{
	case 'a': pixel = "::o::  :o:o:  o:::o  ooooo  o:::o  o:::o  o:::o"; break;
	case 'b': pixel = "oooo:  o:::o  o:::o  oooo:  o:::o  o:::o  oooo:"; break;
	case 'c': pixel = ":ooo:  o:::o  o::::  o::::  o::::  o:::o  :ooo:"; break;
	case 'd': pixel = "ooo::  o::o:  o:::o  o:::o  o:::o  o::o:  ooo::"; break;
	case 'e': pixel = "ooooo  o::::  o::::  ooo::  o::::  o::::  ooooo"; break;
	case 'f': pixel = "ooooo  o::::  o::::  ooo::  o::::  o::::  o::::"; break;
	case 'g': pixel = ":ooo:  o:::o  o::::  o:ooo  o:::o  o:::o  :oooo"; break;
	case 'h': pixel = "o:::o  o:::o  o:::o  ooooo  o:::o  o:::o  o:::o"; break;
	case 'i': pixel = ":ooo:  ::o::  ::o::  ::o::  ::o::  ::o::  :ooo:"; break;
	case 'j': pixel = "::::o  ::::o  ::::o  ::::o  o:::o  o:::o  :ooo:"; break;
	case 'k': pixel = "o:::o  o::o:  o:o::  ooo::  o::o:  o:::o  o:::o"; break;
	case 'l': pixel = "o::::  o::::  o::::  o::::  o::::  o::::  ooooo"; break;
	case 'm': pixel = "o:::o  oo:oo  o:o:o  o:o:o  o:o:o  o:::o  o:::o"; break;
	case 'n': pixel = "o:::o  oo::o  o:o:o  o::oo  o:::o  o:::o  o:::o"; break;
	case 'o': pixel = ":ooo:  o:::o  o:::o  o:::o  o:::o  o:::o  :ooo:"; break;
	case 'p': pixel = "oooo:  o:::o  o:::o  oooo:  o::::  o::::  o::::"; break;
	case 'q': pixel = ":ooo:  o:::o  o:::o  o:::o  o:o:o  o::oo  :oooo"; break;
	case 'r': pixel = "oooo:  o:::o  o:::o  oooo:  o:::o  o:::o  o:::o"; break;
	case 's': pixel = ":ooo:  o:::o  o::::  :ooo:  ::::o  o:::o  :ooo:"; break;
	case 't': pixel = "ooooo  ::o::  ::o::  ::o::  ::o::  ::o::  ::o::"; break;
	case 'u': pixel = "o:::o  o:::o  o:::o  o:::o  o:::o  o:::o  :ooo:"; break;
	case 'v': pixel = "o:::o  o:::o  o:::o  o:::o  o:::o  :o:o:  ::o::"; break;
	case 'w': pixel = "o:::o  o:::o  o:::o  o:o:o  o:o:o  o:o:o  oo:oo"; break;
	case 'x': pixel = "o:::o  o:::o  :o:o:  ::o::  :o:o:  o:::o  o:::o"; break;
	case 'y': pixel = "o:::o  o:::o  :o:o:  ::o::  ::o::  ::o::  ::o::"; break;
	case 'z': pixel = "ooooo  ::::o  :::o:  ::o::  :o:::  o::::  ooooo"; break;
	default: return;
	}

	// create pixels
	for (unsigned int i = 0; i < pixel.size(); i++)
	{
		if (pixel[i] != ':' && pixel[i] != ' ')
		{
			for (int y = 0; y <= TEXTSIZE - 1; y++)
			{
				for (int x = 0; x <= TEXTSIZE - 1; x++)
				{
					sandbox.Menu.push_back(new Update(sandbox, type, glm::ivec2(pos.x + x + (TEXTSIZE * (i % width)), pos.y + y + (TEXTSIZE * (int)floor((float)i / (float)width))), false));
				}
			}
		}
	}
}