#include "Updates.h"
#include "Main.h"

Update::Update(Sandbox& p_sandbox, Types p_type, glm::ivec2 p_pos, bool InArray) :
	pos(p_pos),
	type(p_type),
	CloneType(Types::Air)
{
	if (InArray)
		p_sandbox.grid[p_pos.y * SCRWIDTH + p_pos.x] = this;
}

void Update::Move(Sandbox& sandbox, Direction direction)
{
	int x = 0;
	int y = 0;

	switch (direction)
	{
	case Direction::Up: y--; break;
	case Direction::Down: y++; break;
	case Direction::Left: x--; break;
	case Direction::Right: x++; break;
	case Direction::ULeft: y--; x--; break;
	case Direction::URight: y--; x++; break;
	case Direction::BLeft: y++; x--; break;
	case Direction::BRight: y++; x++; break;
	}

	// move if empty, otherwise swap
	Update* cell = sandbox.grid[(pos.y + y) * SCRWIDTH + pos.x + x];
	sandbox.grid[pos.y * SCRWIDTH + pos.x] = cell;
	sandbox.grid[(pos.y + y) * SCRWIDTH + pos.x + x] = this;
	pos += glm::ivec2(x, y);
	if (cell != nullptr) cell->pos -= glm::ivec2(x, y);

	// delete if on the sides
	if (pos.x == 0 || pos.x == SCRWIDTH - 1 || pos.y == 0 || pos.y == SCRHEIGHT - 1)
		QueueDelete = true;
}

void Update::UpdateCell(Sandbox& sandbox)
{
	switch (type)
	{
	case Types::Block: return;
	case Types::Sand: Sand(sandbox); break;
	case Types::Water: Water(sandbox); break;
	case Types::Lava: Lava(sandbox); break;
	case Types::Stone: Stone(sandbox); break;
	case Types::Clone: Clone(sandbox); break;
	case Types::Ice: Ice(sandbox); break;
	case Types::Cloud: Cloud(sandbox); break;
	case Types::Fire: Fire(sandbox); break;
	case Types::Oil: Oil(sandbox); break;
	case Types::Powder: Powder(sandbox); break;
	case Types::Acid: Acid(sandbox); break;
	case Types::Gas: Gas(sandbox); break;
	}
}

glm::vec3 Update::RenderCell()
{
	switch (type)
	{
	case Types::Block: return glm::vec3(50, 50, 50);
	case Types::Sand: return glm::vec3(255, 255, 0);
	case Types::Water: return glm::vec3(0, 50, 255);
	case Types::Lava: return glm::vec3(255, 100, 0);
	case Types::Stone: return glm::vec3(100, 100, 100);
	case Types::Clone: return glm::vec3(120, 70, 0);
	case Types::Ice: return glm::vec3(225, 225, 255);
	case Types::Cloud: return glm::vec3(200, 200, 200);
	case Types::Fire: return glm::vec3(255, 50, 0);
	case Types::Oil: return glm::vec3(80, 40, 20);
	case Types::Powder: return glm::vec3(255, 220, 0);
	case Types::Acid: return glm::vec3(200, 255, 0);
	case Types::Gas: return glm::vec3(200, 130, 120);
	}
	return glm::vec3(0, 0, 0);
}

void Update::Sand(Sandbox& sandbox)
{
	if (pos.y != SCRHEIGHT - 1)
	{
		Update* below = sandbox.grid[(pos.y + 1) * SCRWIDTH + pos.x];
		if (below == nullptr || below->type == Types::Water || below->type == Types::Oil)
		{
			Move(sandbox, Direction::Down);
			return;
		}

		Update* bleft = sandbox.grid[(pos.y + 1) * SCRWIDTH + pos.x - 1];
		Update* bright = sandbox.grid[(pos.y + 1) * SCRWIDTH + pos.x + 1];
		if (bleft != nullptr && bleft->type != Types::Water && bleft->type != Types::Oil &&
			bright != nullptr && bright->type != Types::Water && bright->type != Types::Oil)
			return;

		bool chance = (bool)(rand() % 2);

		if (chance && pos.x != 0)
		{
			if (bleft == nullptr || bleft->type == Types::Water || bleft->type == Types::Oil)
			{
				Move(sandbox, Direction::BLeft);
				return;
			}
		}

		if (!chance && pos.x != SCRWIDTH - 1)
		{
			if (bright == nullptr || bright->type == Types::Water || bright->type == Types::Oil)
			{
				Move(sandbox, Direction::BRight);
				return;
			}
		}
	}
	else
		QueueDelete = true;
}

void Update::Water(Sandbox& sandbox)
{
	if (pos.y != SCRHEIGHT - 1)
	{
		Update* below = sandbox.grid[(pos.y + 1) * SCRWIDTH + pos.x];

		if (below == nullptr || below->type == Types::Oil || below->type == Types::Cloud)
		{
			Move(sandbox, Direction::Down);
			return;
		}
	}
	else
		QueueDelete = true;

	Update* left = sandbox.grid[pos.y * SCRWIDTH + pos.x - 1];
	Update* right = sandbox.grid[pos.y * SCRWIDTH + pos.x + 1];
	if (left != nullptr && left->type != Types::Oil && left->type != Types::Cloud &&
		right != nullptr && right->type != Types::Oil && right->type != Types::Cloud)
		return;

	bool chance = (bool)(rand() % 2);

	if (chance && pos.x != 0)
	{
		if (left == nullptr || left->type == Types::Oil || left->type == Types::Cloud)
		{
			Move(sandbox, Direction::Left);
			return;
		}
	}

	if (!chance && pos.x != SCRWIDTH - 1)
	{
		if (right == nullptr || right->type == Types::Oil || right->type == Types::Cloud)
		{
			Move(sandbox, Direction::Right);
			return;
		}
	}
}

void Update::Lava(Sandbox& sandbox)
{
	bool convertable = (rand() % 10) == 0 ? true : false;
	bool moved = false;

	if (convertable && pos.y != 0)
	{
		Update* above = sandbox.grid[(pos.y - 1) * SCRWIDTH + pos.x];

		// sand -> lava
		if (above != nullptr && above->type == Types::Sand)
			above->type = Types::Lava;

		// water -> stone
		if (above != nullptr && above->type == Types::Water)
		{
			above->type = Types::Stone;
			QueueDelete = true;
			return;
		}
	}
	else if (pos.y == 0)
		QueueDelete = true;

	if (pos.y != SCRHEIGHT - 1)
	{
		Update* below = sandbox.grid[(pos.y + 1) * SCRWIDTH + pos.x];

		if (convertable)
		{
			// sand -> lava
			if (convertable && below != nullptr && below->type == Types::Sand)
				below->type = Types::Lava;

			// water -> stone
			if (below != nullptr && below->type == Types::Water)
			{
				below->type = Types::Stone;
				QueueDelete = true;
				return;
			}
		}

		// Move
		if (!moved && (below == nullptr || below->type == Types::Cloud))
		{
			Move(sandbox, Direction::Down);
			moved = true;
		}
	}

	Update* left = sandbox.grid[pos.y * SCRWIDTH + pos.x - 1];
	Update* right = sandbox.grid[pos.y * SCRWIDTH + pos.x + 1];
	if (left != nullptr && left->type != Types::Stone && left->type != Types::Cloud &&
		right != nullptr && right->type != Types::Stone && right->type != Types::Cloud)
		moved = true;

	bool chance = false;
	if (!moved)
		chance = (bool)(rand() % 2);

	if (pos.x != 0)
	{

		if (convertable)
		{
			// sand -> lava
			if (convertable && left != nullptr && left->type == Types::Sand)
				left->type = Types::Lava;

			// water -> stone
			if (left != nullptr && left->type == Types::Water)
			{
				left->type = Types::Stone;
				QueueDelete = true;
				return;
			}
		}

		// Move
		if (!moved && chance && (left == nullptr || left->type == Types::Stone || left->type == Types::Cloud))
		{
			Move(sandbox, Direction::Left);
			moved = true;
		}
	}

	if (pos.x != SCRWIDTH - 1)
	{

		if (convertable)
		{
			// sand -> lava
			if (convertable && right != nullptr && right->type == Types::Sand)
				right->type = Types::Lava;

			// water -> stone
			if (right != nullptr && right->type == Types::Water)
			{
				right->type = Types::Stone;
				QueueDelete = true;
				return;
			}
		}

		// Move
		if (!moved && !chance && (right == nullptr || right->type == Types::Stone || right->type == Types::Cloud))
		{
			Move(sandbox, Direction::Right);
			moved = true;
		}
	}
}

void Update::Stone(Sandbox& sandbox)
{
	if (pos.y != SCRHEIGHT - 1)
	{
		Update* below = sandbox.grid[(pos.y + 1) * SCRWIDTH + pos.x];

		if (below == nullptr || below->type == Types::Water || below->type == Types::Lava || below->type == Types::Oil || below->type == Types::Cloud)
		{
			Move(sandbox, Direction::Down);
			return;
		}
	}
	else
		QueueDelete = true;
}

void Update::Clone(Sandbox& sandbox)
{
	if (CloneType == Types::Air)
	{
		// check pixels for type
		for (int y = pos.y - 1; y <= pos.y + 1; y++)
		{
			for (int x = pos.x - 1; x <= pos.x + 1; x++)
			{
				if (x >= 0 && x < SCRWIDTH && y >= 0 && y < SCRHEIGHT)
				{
					Update* cell = sandbox.grid[y * SCRWIDTH + x];
					if (cell != nullptr && cell->type != Types::Clone && cell->type != Types::Block)
					{
						CloneType = cell->type;
						return;
					}
				}
			}
		}
	}
	else
	{
		// spawn pixels
		Direction direction = Direction(rand() % 8);
		int x = pos.x;
		int y = pos.y;
		switch (direction)
		{
		case Direction::Up: y--; break;
		case Direction::Down: y++; break;
		case Direction::Left: x--; break;
		case Direction::Right: x++; break;
		case Direction::ULeft: y--; x--; break;
		case Direction::URight: y--; x++; break;
		case Direction::BLeft: y++; x--; break;
		case Direction::BRight: y++; x++; break;
		}

		if (x >= 0 && x < SCRWIDTH && y >= 0 && y < SCRHEIGHT)
		{
			if (sandbox.grid[y * SCRWIDTH + x] == nullptr)
			{
				sandbox.cells.push_back(new Update(sandbox, CloneType, glm::ivec2(x, y), true));
			}
		}
	}
}

void Update::Ice(Sandbox& sandbox)
{
	int chance = rand() % 10;

	if (pos.y != 0)
	{
		Update* above = sandbox.grid[(pos.y - 1) * SCRWIDTH + pos.x];

		// water & cloud -> ice
		if (chance == 0 && above != nullptr && (above->type == Types::Water || above->type == Types::Cloud))
			above->type = Types::Ice;

		// lava & fire -> water
		if (!QueueDelete && above != nullptr && (above->type == Types::Lava || above->type == Types::Fire))
		{
			above->type = Types::Water;
			QueueDelete = true;
		}
	}

	if (pos.y != SCRHEIGHT - 1)
	{
		Update* below = sandbox.grid[(pos.y + 1) * SCRWIDTH + pos.x];

		// water & cloud -> ice
		if (chance == 0 && below != nullptr && (below->type == Types::Water || below->type == Types::Cloud))
			below->type = Types::Ice;

		// lava & fire -> water
		if (!QueueDelete && below != nullptr && (below->type == Types::Lava || below->type == Types::Fire))
		{
			below->type = Types::Water;
			QueueDelete = true;
		}
	}

	if (pos.x != 0)
	{
		Update* left = sandbox.grid[pos.y * SCRWIDTH + pos.x - 1];

		// water & cloud -> ice
		if (chance == 0 && left != nullptr && (left->type == Types::Water || left->type == Types::Cloud))
			left->type = Types::Ice;

		// lava & fire -> water
		if (!QueueDelete && left != nullptr && (left->type == Types::Lava || left->type == Types::Fire))
		{
			left->type = Types::Water;
			QueueDelete = true;
		}
	}

	if (pos.x != SCRWIDTH - 1)
	{
		Update* right = sandbox.grid[pos.y * SCRWIDTH + pos.x + 1];

		// water & cloud -> ice
		if (chance == 0 && right != nullptr && (right->type == Types::Water || right->type == Types::Cloud))
			right->type = Types::Ice;

		// lava & fire -> water
		if (!QueueDelete && right != nullptr && (right->type == Types::Lava || right->type == Types::Fire))
		{
			right->type = Types::Water;
			QueueDelete = true;
		}
	}
}

void Update::Cloud(Sandbox& sandbox)
{
	int chance = rand() % 3;

	if (pos.y != 0)
	{
		Update* above = sandbox.grid[(pos.y - 1) * SCRWIDTH + pos.x];

		if (chance == 0 && above == nullptr)
			Move(sandbox, Direction::Up);
		else if (above != nullptr && above->type == Types::Block)
			type = Types::Water;
	}
	else
		QueueDelete = true;

	if (pos.x != 0)
	{
		Update* left = sandbox.grid[pos.y * SCRWIDTH + pos.x - 1];
		if (chance == 1 && left == nullptr)
			Move(sandbox, Direction::Left);
		else if (left != nullptr && left->type == Types::Block)
			type = Types::Water;
	}
	else
		type = Types::Water;

	if (pos.x != SCRWIDTH - 1)
	{
		Update* right = sandbox.grid[pos.y * SCRWIDTH + pos.x + 1];
		if (chance == 2 && right == nullptr)
			Move(sandbox, Direction::Right);
		else if (right != nullptr && right->type == Types::Block)
			type = Types::Water;
	}
	else
		type = Types::Water;
}

void Update::Fire(Sandbox& sandbox)
{
	int Delete = rand() % 50;
	if (Delete == 0)
	{
		QueueDelete = true;
		return;
	}

	if (pos.y != 0)
	{
		Update* above = sandbox.grid[(pos.y - 1) * SCRWIDTH + pos.x];

		if (above == nullptr)
			Move(sandbox, Direction::Up);
		else if (above->type == Types::Water)
		{
			QueueDelete = true;
			return;
		}
	}
	else
		QueueDelete = true;

	Update* left = sandbox.grid[pos.y * SCRWIDTH + pos.x - 1];
	Update* right = sandbox.grid[pos.y * SCRWIDTH + pos.x + 1];
	if (left != nullptr && right != nullptr)
		return;

	bool MoveLeft = (bool)(rand() % 2);

	if (pos.x != 0)
	{
		if (MoveLeft && left == nullptr)
			Move(sandbox, Direction::Left);
	}

	if (pos.x != SCRWIDTH - 1)
	{
		if (!MoveLeft && right == nullptr)
			Move(sandbox, Direction::Right);
	}
}

void Update::Oil(Sandbox& sandbox)
{
	bool convertable = (rand() % 8) < 2 ? true : false;
	bool moved = false;

	if (convertable && pos.y != 0)
	{
		Update* above = sandbox.grid[(pos.y - 1) * SCRWIDTH + pos.x];

		if (above != nullptr && (above->type == Types::Fire || above->type == Types::Lava))
		{
			type = Types::Fire;
			return;
		}
	}

	if (pos.y != SCRHEIGHT - 1)
	{
		Update* below = sandbox.grid[(pos.y + 1) * SCRWIDTH + pos.x];

		if (convertable && below != nullptr && (below->type == Types::Fire || below->type == Types::Lava))
		{
			type = Types::Fire;
			return;
		}

		if (moved == false && below == nullptr)
		{
			Move(sandbox, Direction::Down);
			moved = true;
		}
	}
	else
		QueueDelete = true;

	Update* left = sandbox.grid[pos.y * SCRWIDTH + pos.x - 1];
	Update* right = sandbox.grid[pos.y * SCRWIDTH + pos.x + 1];
	if (left != nullptr && left->type != Types::Fire && left->type != Types::Lava &&
		right != nullptr && right->type != Types::Fire && right->type != Types::Lava)
		moved = true;

	bool chance = false;
	if (!moved)
		chance = (bool)(rand() % 2);

	if (pos.x != 0)
	{

		if (convertable && left != nullptr && (left->type == Types::Fire || left->type == Types::Lava))
		{
			type = Types::Fire;
			return;
		}

		if (chance && moved == false && left == nullptr)
		{
			Move(sandbox, Direction::Left);
			moved = true;
		}
	}

	if (pos.x != SCRWIDTH - 1)
	{

		if (convertable && right != nullptr && (right->type == Types::Fire || right->type == Types::Lava))
		{
			type = Types::Fire;
			return;
		}

		if (!chance && moved == false && right == nullptr)
		{
			Move(sandbox, Direction::Right);
			moved = true;
		}
	}
}

void Update::Powder(Sandbox& sandbox)
{
	bool convertable = (rand() % 8) < 5 ? true : false;

	if (convertable && pos.y != 0)
	{
		Update* above = sandbox.grid[(pos.y - 1) * SCRWIDTH + pos.x];

		if (above != nullptr && (above->type == Types::Fire || above->type == Types::Lava))
			type = Types::Fire;
	}

	if (pos.y != SCRHEIGHT - 1)
	{
		Update* below = sandbox.grid[(pos.y + 1) * SCRWIDTH + pos.x];

		if (convertable && below != nullptr && (below->type == Types::Fire || below->type == Types::Lava))
			type = Types::Fire;

		if (below == nullptr || below->type == Types::Water || below->type == Types::Oil)
			Move(sandbox, Direction::Down);
	}
	else
		QueueDelete = true;

	if (pos.x != 0)
	{
		Update* left = sandbox.grid[pos.y * SCRWIDTH + pos.x - 1];

		if (convertable && left != nullptr && (left->type == Types::Fire || left->type == Types::Lava))
			type = Types::Fire;
	}

	if (pos.x != SCRWIDTH - 1)
	{
		Update* right = sandbox.grid[pos.y * SCRWIDTH + pos.x + 1];

		if (convertable && right != nullptr && (right->type == Types::Fire || right->type == Types::Lava))
			type = Types::Fire;
	}
}

void Update::Acid(Sandbox& sandbox)
{
	bool moved = false;

	if (pos.y != 0)
	{
		Update* above = sandbox.grid[(pos.y - 1) * SCRWIDTH + pos.x];

		if (above != nullptr &&
			above->type != Types::Acid &&
			above->type != Types::Block &&
			above->type != Types::Clone &&
			above->type != Types::Fire)
		{
			above->QueueDelete = true;
			QueueDelete = true;
			return;
		}
	}

	if (pos.y != SCRHEIGHT - 1)
	{
		Update* below = sandbox.grid[(pos.y + 1) * SCRWIDTH + pos.x];

		if (below != nullptr &&
			below->type != Types::Acid &&
			below->type != Types::Block &&
			below->type != Types::Clone &&
			below->type != Types::Fire)
		{
			below->QueueDelete = true;
			QueueDelete = true;
			return;
		}

		if (moved == false && below == nullptr)
		{
			Move(sandbox, Direction::Down);
			moved = true;
		}
	}
	else
		QueueDelete = true;

	Update* left = sandbox.grid[pos.y * SCRWIDTH + pos.x - 1];
	Update* right = sandbox.grid[pos.y * SCRWIDTH + pos.x + 1];
	if (left != nullptr && right != nullptr)
		moved = true;

	bool chance = false;
	if (!moved)
		chance = (bool)(rand() % 2);

	if (pos.x != 0)
	{

		if (left != nullptr &&
			left->type != Types::Acid &&
			left->type != Types::Block &&
			left->type != Types::Clone &&
			left->type != Types::Fire)
		{
			left->QueueDelete = true;
			QueueDelete = true;
			return;
		}

		if (chance && moved == false && left == nullptr)
		{
			Move(sandbox, Direction::Left);
			moved = true;
		}
	}

	if (pos.x != SCRWIDTH - 1)
	{

		if (right != nullptr &&
			right->type != Types::Acid &&
			right->type != Types::Block &&
			right->type != Types::Clone &&
			right->type != Types::Fire)
		{
			right->QueueDelete = true;
			QueueDelete = true;
			return;
		}

		if (!chance && moved == false && right == nullptr)
		{
			Move(sandbox, Direction::Right);
			moved = true;
		}
	}
}

void Update::Gas(Sandbox& sandbox)
{
	bool convertable = (rand() % 8) < 6 ? true : false;
	bool moved = false;

	if (pos.y != 0)
	{
		Update* above = sandbox.grid[(pos.y - 1) * SCRWIDTH + pos.x];

		if (convertable && above != nullptr && (above->type == Types::Fire || above->type == Types::Lava))
		{
			type = Types::Fire;
			return;
		}

		if (!moved && above == nullptr)
		{
			Move(sandbox, Direction::Up);
		}
	}
	else
	{
		QueueDelete = true;
		return;
	}

	if (convertable && pos.y != SCRHEIGHT - 1)
	{
		Update* below = sandbox.grid[(pos.y + 1) * SCRWIDTH + pos.x];

		if (convertable && below != nullptr && (below->type == Types::Fire || below->type == Types::Lava))
		{
			type = Types::Fire;
			return;
		}
	}

	Update* left = sandbox.grid[pos.y * SCRWIDTH + pos.x - 1];
	Update* right = sandbox.grid[pos.y * SCRWIDTH + pos.x + 1];
	bool chance = false;
	if (!moved)
	{
		if (left != nullptr && right != nullptr)
			moved = true;

		chance = (bool)(rand() % 2);
	}

	if (pos.x != 0)
	{
		if (convertable && left != nullptr && (left->type == Types::Fire || left->type == Types::Lava))
		{
			type = Types::Fire;
			return;
		}

		if (!moved && chance && left == nullptr)
		{
			Move(sandbox, Direction::Left);
			moved = true;
		}
	}

	if (pos.x != SCRWIDTH - 1)
	{
		if (convertable && right != nullptr && (right->type == Types::Fire || right->type == Types::Lava))
		{
			type = Types::Fire;
			return;
		}

		if (!moved && !chance && right == nullptr)
		{
			Move(sandbox, Direction::Right);
			moved = true;
		}
	}
}